----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    08:57:48 08/26/2014 
-- Design Name: 
-- Module Name:    microcontroleur - microcontroleur_architecture 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity micrcontroleur is
    Port ( clk : in  STD_LOGIC;
           Rst : in  STD_LOGIC;
           sw : in STD_LOGIC_VECTOR (7 downto 0);
			  HServo,VServo : out std_logic;
			  sonar_trig : out STD_LOGIC;
           sonar_echo : in  STD_LOGIC;
           lcd_e  : out std_logic;
           lcd_rs : out std_logic;
           lcd_rw : out std_logic;
           lcd_db : out std_logic_vector(7 downto 4);
           Q_PORTA : out STD_LOGIC_VECTOR (7 downto 0)
           );
end micrcontroleur;

architecture microcontroleur_architecture of micrcontroleur is
--Registres et PORTs de l'ATTiny861
constant OCR1A : std_logic_vector(5 downto 0) := "101101";
constant OCR1B : std_logic_vector(5 downto 0) := "101100";
constant PORTA : std_logic_vector(5 downto 0) := "011011";
constant DDRA : std_logic_vector(5 downto 0) := "011010";
constant PINA : std_logic_vector(5 downto 0) := "011001";
constant PORTB : std_logic_vector(5 downto 0) := "011000";
constant DDRB : std_logic_vector(5 downto 0) := "010111";
constant PINB : std_logic_vector(5 downto 0) := "010110";
constant ADCH : std_logic_vector(5 downto 0) := "000101";
constant ADCL : std_logic_vector(5 downto 0) := "000100";
--Registres non présents dans l'ATTiny861
constant UDR : std_logic_vector(5 downto 0) := "000011";
constant UCSRA : std_logic_vector(5 downto 0) := "000010";
constant UCSRB : std_logic_vector(5 downto 0) := "000001";
component mcu_core is
		Port (
			Clk	: in std_logic;
			Rst	: in std_logic; -- Reset core when Rst='1'
			En		: in std_logic; -- CPU stops when En='0', could be used to slow down cpu to save power
			-- PM
			PM_A		: out std_logic_vector(15 downto 0);
			PM_Drd	: in std_logic_vector(15 downto 0);
			-- DM
			DM_A		: out std_logic_vector(15 downto 0); -- 0x00 - xxxx
			DM_Areal	: out std_logic_vector(15 downto 0); -- 0x60 - xxxx (same as above + io-adr offset)
			DM_Drd	: in std_logic_vector(7 downto 0);
			DM_Dwr	: out std_logic_vector(7 downto 0);
			DM_rd		: out std_logic;
			DM_wr		: out std_logic;
			-- IO
			IO_A		: out std_logic_vector(5 downto 0); -- 0x00 - 0x3F
			IO_Drd	: in std_logic_vector(7 downto 0);
			IO_Dwr	: out std_logic_vector(7 downto 0);
			IO_rd		: out std_logic;
			IO_wr		: out std_logic;
			-- OTHER
		   OT_FeatErr	: out std_logic; -- Feature error! (Unhandled part of instruction)
		   OT_InstrErr	: out std_logic -- Instruction error! (Unknown instruction)
		);
	end component mcu_core;
	--PM
	component pm  is
		Port (
				Clk	: in std_logic;
				rst	: in std_logic; -- Reset when Rst='1'
				-- PM
				PM_A		: in std_logic_vector(15 downto 0);
				PM_Drd	: out std_logic_vector(15 downto 0)
		);
	end component pm;	

  component dm is
    Port ( clk : in  STD_LOGIC;
           addr : in  STD_LOGIC_VECTOR (15 downto 0);
           dataread : out  STD_LOGIC_VECTOR (7 downto 0);
           datawrite : in  STD_LOGIC_VECTOR (7 downto 0);
           rd : in  STD_LOGIC;
           wr : in  STD_LOGIC);
  end component dm;
  
  component SERVO is
	port(
		MCLK		: in	std_logic;
		nRST		: in	std_logic;
		DATA		: in	std_logic_vector(7 downto 0);
		SERVO		: out	std_logic
	);
  end component SERVO;
  
  component sonar is
    Port ( clk        : in  STD_LOGIC;
           sonar_trig : out STD_LOGIC;
           sonar_echo : in  STD_LOGIC;
			  waiting    : out  std_logic;
           dist_cm  : out std_logic_vector(15 downto 0));
  end component sonar;
  
  	signal PM_A		: std_logic_vector(15 downto 0);
	signal PM_Drd	: std_logic_vector(15 downto 0);
	-- DM
	signal DM_A			: std_logic_vector(15 downto 0); -- 0x00 - xxxx
	signal DM_Areal	: std_logic_vector(15 downto 0); -- 0x60 - xxxx (same as above + io-adr offset)
	signal DM_Drd		: std_logic_vector(7 downto 0);
	signal DM_Dwr		: std_logic_vector(7 downto 0);
	signal DM_rd		: std_logic;
	signal DM_wr		: std_logic;
	-- IO
	signal IO_A		: std_logic_vector(5 downto 0); -- 0x00 - 0x3F
	signal IO_Drd	: std_logic_vector(7 downto 0);
	signal IO_Dwr	: std_logic_vector(7 downto 0);
	signal IO_rd	: std_logic;
	signal IO_wr	: std_logic;

	signal s_PORTB	: std_logic_vector(7 downto 0);
--	signal IO_DrdB	: std_logic_vector(7 downto 0);
   signal pan_data : std_logic_vector(7 downto 0);
	signal tilt_data : std_logic_vector(7 downto 0);
	signal dist_cmL : std_logic_vector(7 downto 0);
	signal s_waiting : std_logic;
begin
	core : mcu_core Port map (
		Clk	=> clk,
		Rst	=> Rst,
		En		=> '1',
		-- PM
		PM_A		=> PM_A,
		PM_Drd	=> PM_Drd,
		-- DM
		DM_A		=> DM_A,
		DM_Areal	=> DM_Areal,
		DM_Drd	=> DM_Drd,
		DM_Dwr	=> DM_Dwr,
		DM_rd		=> DM_rd,
		DM_wr		=> DM_wr,
		-- IO
		IO_A		=> IO_A,
		IO_Drd	=> IO_Drd,
		IO_Dwr	=> IO_Dwr,
		IO_rd		=> IO_rd,
		IO_wr		=> IO_wr,
		-- OTHER
		OT_FeatErr => open,
		OT_InstrErr	=> open
	);

	prgmem : pm port map (
			Clk	=> clk,
			Rst	=> '0',
			-- PM
			PM_A		=> PM_A,
			PM_Drd	=> PM_Drd
	);
	
	datamem : dm port map (
           clk => clk,
           addr => DM_A,
           dataread  => DM_Drd,
           datawrite => DM_Dwr,
           rd => DM_rd, 
           wr =>	DM_wr
	);
	
	pan_servo : SERVO port map (
		MCLK	=> clk,
		nRST	=> NOT Rst,
		DATA	=> pan_data,
		SERVO	=> HServo
		);
	
	tilt_servo : SERVO port map (
		MCLK	=> clk,
		nRST	=> NOT Rst,
		DATA	=> tilt_data,
		SERVO	=> VServo
		);
 	ic_sonar : sonar Port map( 
	        clk  => clk,
           sonar_trig => sonar_trig,
           sonar_echo => sonar_echo,
			  waiting  => s_waiting,
           dist_cm(7 downto 0) => dist_cmL
			  );
			  
   iowr: process(CLK)
    begin
        if (rising_edge(CLK)) then
            if (IO_wr = '1') then
                case IO_A is
		 -- addresses for tiny861 device (use io.h).
       --
                    when PORTA  => -- PORTA=X"1B" (0X3B)
                         Q_PORTA <= IO_Dwr;
                    when  PORTB => -- PORTB=X"18" (0X38)
						       s_PORTB <= IO_Dwr;
						  when DDRA =>
						       pan_data <= IO_Dwr;
						  when DDRB => 
						       tilt_data <= IO_Dwr;
                    when others =>
                end case;
            end if;
        end if;
    end process;
	 lcd_db <= s_PORTB(7 downto 4);
	 lcd_e <= s_PORTB(1);
	 lcd_rs <= s_PORTB(2); 
	 lcd_rw <= s_PORTB(3); 
-- IO read process
--
    iord: process(IO_rd,IO_A)
    begin
        -- addresses for tinyX6 device (use iom8.h).
        --
        if IO_rd = '1' then
          case IO_A is
            when  PINA => IO_Drd <= sw;  -- PINA=X"19" (0X39)
            when  PORTB => IO_Drd <= s_PORTB;  -- PORTB=X"18" (0X38)
				when  PINB => IO_Drd <= dist_cmL; -- resultat sonar
				when  DDRB => IO_Drd <= "0000000" & s_waiting;
            when others => IO_Drd <= X"AA";
          end case;
       end if;
    end process;

end microcontroleur_architecture;
