//      essai.c
//      
//      Copyright 2014 Michel Doussot <michel@mustafar>
//      
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.

#include <avr/io.h>
#define RS 		0x04
#define E		0x02
#define RW 		0x08
#define DATAS  	0xF0

// unite approximative 2us
void delai(unsigned long int delai) {
	volatile long int i=0;
	for(i=0;i<delai;i+=1);
} 

// portB :
// b7 b6 b5 b4 b3 b2 b1 b0
// D3 D2 D1 D0 RW RS  E CE0
void rs_haut(void) {
   PORTB = PORTB | RS;
}

void rs_bas(void) {
   PORTB = PORTB & ~RS; 
}

void rw_haut(void) {
   PORTB = PORTB | RW;
}

void rw_bas(void) {
   PORTB = PORTB & ~RW; 
}

void e_haut(void) {
   PORTB = PORTB | E;
   delai(8);
}

void e_bas(void) {
    PORTB = PORTB & ~E; 
    delai(8);
}

void e_puls(void) {
   e_haut();
   e_bas();
}

void ecris_4(unsigned char valeur) {
	unsigned char v;
	v = (valeur << 4) & DATAS;
	PORTB = PORTB & ~DATAS ;
	PORTB = PORTB | v ;
    e_puls();
}

void ecris_8(unsigned char valeur) {
	unsigned char v;
	v = valeur & DATAS;
	PORTB = PORTB & ~DATAS ;
	PORTB = PORTB | v ;
    e_puls();
	v = (valeur << 4) & DATAS;
	PORTB = PORTB & ~DATAS ;
	PORTB = PORTB | v ;
    e_puls();    
}


void setup() {

   PORTB = 0;
   delai(6000);
   ecris_4(0x03);
   delai(1600);
   ecris_4(0x03);
   delai(800);
   ecris_4(0x03);
   delai(800);
   ecris_4(0x02);
   delai(40);
   ecris_4(0x02);
   ecris_4(0x08);
   delai(40);
   ecris_4(0x00);
   ecris_4(0x06);
   delai(40);
   ecris_4(0x00);
   ecris_4(0x0C);
   delai(40);
   ecris_4(0x00);
   ecris_4(0x01);
   delai(800);   
}

void writecar(char car) {
	rs_haut();
	ecris_8((unsigned char)car);
}

void writestr(char *chaine) {
	rs_haut();
	while (*chaine) {
		ecris_8((unsigned char)*chaine++);
	}
}

void command (uint8_t value) {
  rs_bas();
  ecris_8(value);
}

#define LCD_SETDDRAMADDR 0x80

void setCursor(uint8_t col, uint8_t row) {
  col = col & 0x0F; // %16
  row = row & 0x01; // %2
  command(LCD_SETDDRAMADDR | (col + 0x40*row));
  delai(100);
}

#define LCD_CLEARDISPLAY 0x01

void clearScreen() {
  command(LCD_CLEARDISPLAY);  // clear display, set cursor position to zero
  delai(1000);  // this command takes a long time!
}

#define LCD_SETCGRAMADDR 0x40
// Allows us to fill the first 8 CGRAM locations
// with custom characters
void createChar(uint8_t location, uint8_t charmap[]) {
  uint8_t i;
  location &= 0x7; // we only have 8 locations 0-7
  command(LCD_SETCGRAMADDR | (location << 3));
  for (i=0; i<8; i++) {
	  writecar((char)charmap[i]);delai(100);
  }
}

uint8_t bar0[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000
};

uint8_t bar1[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b11111
};

uint8_t bar2[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b11111,
	0b11111
};

uint8_t bar3[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b11111,
	0b11111,
	0b11111
};

uint8_t bar4[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b00000,
	0b00000,
	0b11111,
	0b11111,
	0b11111,
	0b11111
};

uint8_t bar5[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b00000,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111
};

uint8_t bar6[8] = { // from Arduino Library
	0b00000,
	0b00000,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111
};

uint8_t bar7[8] = { // from Arduino Library
	0b00000,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111
};

uint8_t bar8[8] = { // from Arduino Library
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111,
	0b11111
};

int main(void) {
	
	unsigned char tmp,posH, sn,i;
  // setup
	setup();
   	tmp = 0;
        createChar(0,bar0);
		createChar(1,bar1);
		createChar(2,bar2);
		createChar(3,bar3);
		createChar(4,bar4);
		createChar(5,bar5);
		createChar(6,bar6);
		createChar(7,bar7); 
		//createChar(8,bar8);
        clearScreen();
   	writestr("Hello Microsoft ");
    //    setCursor(2,1);//setcursor(h,v)
        //writestr("Hello Linux");
		/*
		for (i=0;i<8;i++)
		  writecar(i); */		
  // loop
	while (1) {
	   setCursor(0,1);
	   for (i=0;i<16;i++) {	
		tmp += 16;	
		posH = PINA;
		DDRA = tmp;//posH;
		delai(100000);
		sn = PINB;
		PORTA = sn;
		writecar(sn>>4);
	   }
	}
	return 0;
}

